package org.braiden.fpm2;

/*
 * Copyright (c) 2010 Braiden Kindt
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 *
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.braiden.fpm2.model.PasswordItem;
import org.braiden.fpm2.util.PropertyUtils;

import android.app.Activity;
import android.app.ListActivity;
import android.app.SearchManager;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.ClipboardManager;
import android.util.Log;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.Filter;
import android.widget.Filterable;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.AdapterView.AdapterContextMenuInfo;

/**
 * Main entry point for application, lists all items in db.
 * 
 * @author braiden
 *
 */
public class PasswordItemListActivity extends ListActivity implements FpmBroadcastReceiver.Listener {
	
	private static final String LAUNCHER_WEB = "Web";

	public final static String EXTRA_ID = "id";
	
	protected final static String TAG = "PasswordListActivity";
	protected final static String CATEGORY_DEFAULT = "org.braiden.fpm2.__DEFAULT__";
	
	private Spinner categoryPicker;
	private boolean isCategoryPickerInitialized;
	private FpmBroadcastReceiver receiver;
	private boolean isPaused;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);    	
    	setContentView(R.layout.password_item_list);
    	
    	// register the adapter for building view for each element of our list
    	BaseAdapter adapter = new FpmCryptListAdapter(getFpmApplication(), getListView());
    	setListAdapter(adapter);
    	getListView().setTextFilterEnabled(true);
    	
    	// get all the categories
    	List<String> allCategories = new ArrayList<String>(20);
    	allCategories.add(getString(R.string.all_category));
    	allCategories.add(getString(R.string.default_category));
    	if (getFpmApplication().isCryptOpen()) {
    		isCategoryPickerInitialized = true;
    		allCategories.addAll(getFpmApplication().getCategories());
    	}
    	
    	// populate the category picker with default values, and any other categories (if availible)
    	categoryPicker = (Spinner) findViewById(R.id.category_picker);
    	final ArrayAdapter<String> categoryData = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, allCategories );
    	categoryData.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    	categoryPicker.setAdapter(categoryData);
    	// on change of picker, we force ListView's filter to be re-applied
    	categoryPicker.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> parentView, View selectedItem, int position, long id) {
				Log.d(TAG, "categoryPicker#onItemSelected(position=" + position + ", id=" + id + ").");
				CharSequence textFilter = getListView().getTextFilter();
				PasswordItemFilter filter = (PasswordItemFilter) ((Filterable) getListAdapter()).getFilter();
				String category = null;
				if (position == 1) {
					category = CATEGORY_DEFAULT;
				} else if (position != 0) {
					category = (String) categoryPicker.getItemAtPosition(position);
				}
				filter.setCategory(category);
				if (getFpmApplication().isCryptOpen()) {
					filter.filter(textFilter);
				}
			}
			
			@Override
			public void onNothingSelected(AdapterView<?> parentView) {
				CharSequence textFilter = getListView().getTextFilter();
				PasswordItemFilter filter = (PasswordItemFilter) ((Filterable) getListAdapter()).getFilter();
				filter.setCategory(null);
				if (getFpmApplication().isCryptOpen()) {
					filter.filter(textFilter);
				}
			}
		});
    	    	
    	receiver = new FpmBroadcastReceiver(this);
    	
    	getListView().setOnCreateContextMenuListener(new View.OnCreateContextMenuListener() {
			@Override
			public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
				AdapterView.AdapterContextMenuInfo info = (AdapterContextMenuInfo) menuInfo;
				PasswordItem item = getFpmApplication().getPasswordItemById(info.id);
				getMenuInflater().inflate(R.menu.password_item_context_menu, menu);
				if (item != null) {
					menu.setHeaderTitle(item.getTitle());
				}
			}
		});
    	
    	
		UnlockCryptActivity.unlockIfRequired(this);
    }
    
	@Override
	protected void onDestroy() {
		super.onDestroy();
		receiver.unregister();
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		isPaused = true;
	}

	@Override
	protected void onResume() {
		super.onResume();
		isPaused = false;
	}

	@Override
	public void onWindowFocusChanged(boolean hasFocus) {
		super.onWindowFocusChanged(hasFocus);
		if (hasFocus) {
			// whenever this view obtains focus, it tries
			// to unlock the fpm store, if the store
			// is not availible, a new acitivity is started
			// to unlock it.
			UnlockCryptActivity.unlockIfRequired(this);
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		android.util.Log.d(TAG, "onActivityResult(requestCode=" + requestCode 
					+ ", resultCode=" + resultCode
					+ ", itent.action=" + (data == null ? null : data.getAction()) + ")");
		
		// if the password unlock activity was canceled,
		// this activity can't continue, finish it.
		if (requestCode == UnlockCryptActivity.UNLOCK_CRYPT_REQUEST_CODE
				&& resultCode == RESULT_CANCELED) {
			finish();
		}
	}

	@Override
	public boolean onSearchRequested() {
		// don't pop-up the search widget if the crypt is locked
		if (getFpmApplication().isCryptOpen()) {
			return super.onSearchRequested();
		} else {
			return false;
		}
	}

	@Override
	protected void onNewIntent(Intent intent) {
		super.onNewIntent(intent);
		// if this acitity receives input from search widget, apply the search string a listView text filter
		if (intent.getAction().equals(Intent.ACTION_SEARCH) && getFpmApplication().isCryptOpen()) {
			String searchString = intent.getStringExtra(SearchManager.QUERY);
			getListView().setFilterText(searchString);
		}
	}

	@Override
	@SuppressWarnings("unchecked")
	public void onFpmUnlock() {
		android.util.Log.d(TAG, "onFpmUnlock()");
		// if the categories have never been initialized,
		// initialize them now.
		if (!isCategoryPickerInitialized) {
			isCategoryPickerInitialized = true;
			for (String cat : getFpmApplication().getCategories()) {
				((ArrayAdapter<String>) categoryPicker.getAdapter()).add(cat);
			}
		}
		((BaseAdapter) getListAdapter()).notifyDataSetChanged();
	}

	@Override
	public void onFpmError(int err) {
		android.util.Log.d(TAG, "onFpmError(" + err + ")");
	}

	@Override
	public void onFpmLock() {
		android.util.Log.d(TAG, "onFpmLock()");
		((BaseAdapter) getListAdapter()).notifyDataSetChanged();
		if (!isPaused) {
			UnlockCryptActivity.unlockIfRequired(this);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.main_menu_prefs:
				startActivity(new Intent(this, FpmPreferencesActivity.class));
				return true;
			case R.id.main_menu_lock:
				getFpmApplication().closeCrypt();
				return true;
			case R.id.main_menu_about:
				startActivity(new Intent(this, AboutActivity.class));
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	@Override
	protected void onListItemClick(ListView listView, View view, int position, long id) {
		boolean launch =  PreferenceManager.getDefaultSharedPreferences(this).getBoolean(
				FpmApplication.PREF_LAUNCH_DEFAULT, false);
		if (launch) {
			launchItem(this, id);
		} else {
			viewItem(this, id);
		}
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
		AdapterView.AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		switch (item.getItemId()) {
			case R.id.context_menu_view:
				viewItem(this, info.id);
				return true;
			case R.id.context_menu_launch:
				launchItem(this, info.id);
				return true;
			case R.id.context_menu_copy_password:
				copyItemProperty(this, info.id, FpmCrypt.PROPERTY_PASSWORD);
				return true;
			case R.id.context_menu_copy_user:
				copyItemProperty(this, info.id, FpmCrypt.PROPERTY_USER);
				return true;
			default:
				return super.onContextItemSelected(item);
		}
	}

	public static void copyItemProperty(Activity activity, long id, String property) {
		FpmApplication app = (FpmApplication) activity.getApplication();
		PasswordItem item = app.getPasswordItemById(id);
		if (item != null) {
			try {
				String value = (String) PropertyUtils.getProperty(item, property);
				if (FpmCrypt.PROPERTY_PASSWORD.equals(property)) {
					value = app.decrypt(value);
				}
				ClipboardManager clipboard = (ClipboardManager) activity.getSystemService(CLIPBOARD_SERVICE);
				clipboard.setText(value);
			} catch (Exception e) {
				Log.w(TAG, "Failed to access property \"" + property + "\" of item id " + id + ".", e);
			}
		}
	}

	public static void launchItem(Activity activity, long id) {		
		PasswordItem item = ((FpmApplication) activity.getApplication()).getPasswordItemById(id);
		if (item != null) {
			try {
				boolean copyPassword = PreferenceManager.getDefaultSharedPreferences(activity).getBoolean(
						FpmApplication.PREF_COPY_PASSWORD, false);
				if (copyPassword) {
					copyItemProperty(activity, id, FpmCrypt.PROPERTY_PASSWORD);
				}
				
				String uriString = item.getUrl();
				
				if (LAUNCHER_WEB.equalsIgnoreCase(item.getLauncher()) && uriString.indexOf("://") < 0)
				{
					uriString = "http://" + uriString;
				}
				
				Uri uri = Uri.parse(uriString);
				Intent intent = new Intent(Intent.ACTION_VIEW, uri);
				activity.startActivity(intent);
			} catch (Exception e) {
				Log.i(TAG, "Cannot launch item (id=" + id + ").", e);
			}
		}
	}

	public static void viewItem(Activity activity, long id) {
    	Intent intent = new Intent(activity, ViewPasswordItemActivity.class);
    	intent.putExtra(EXTRA_ID, id);
    	activity.startActivity(intent);
	}

	protected FpmApplication getFpmApplication() {
		return (FpmApplication) getApplication();
	}
	
	/**
	 * Map our Fpm PasswordItem's to List elements.
	 * 
	 * @author braiden
	 *
	 */
	public static class FpmCryptListAdapter extends BaseAdapter implements Filterable {
    	
    	private LayoutInflater layoutInflater;
    	private FpmApplication app;
    	private List<PasswordItem> data;
    	private Filter filter = null;
    	private ListView listView;
    	
    	public FpmCryptListAdapter(FpmApplication app, ListView listView) {
    		this.layoutInflater = LayoutInflater.from(app);
    		this.listView = listView;
    		this.app = app;
    		if (app.isCryptOpen()) {
    			data = app.getPasswordItems();
    		}
    	}
    	
		@Override
		public void notifyDataSetChanged() {
			// if no data has ever been associated with the
			// view, get all data from the crypt.
			// (if data already associated, don't change it.)
			if (data == null && app.isCryptOpen()) {
				getFilter().filter(listView.getTextFilter());
			} else {
				super.notifyDataSetChanged();
			}
		}

		@Override
		public Filter getFilter() {
			return filter != null ? filter : (filter = new PasswordItemFilter(app, this));
		}

		@Override
		public int getCount() {
			return app.isCryptOpen() && data != null ? data.size() : 0;
		}

		@Override
		public Object getItem(int position) {
			return data.get(position);
		}

		@Override
		public long getItemId(int position) {
			return ((PasswordItem) getItem(position)).getId();
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			ViewHolder viewHolder = null;
			PasswordItem item = (PasswordItem) this.getItem(position);
			
			if (convertView == null) {
				viewHolder = new ViewHolder(); 
				convertView = layoutInflater.inflate(R.layout.password_item_row, null);
				viewHolder.title = (TextView) convertView.findViewById(R.id.passwordItemRowTitle);
				viewHolder.url = (TextView) convertView.findViewById(R.id.passwordItemRowUrl);
				viewHolder.user = (TextView) convertView.findViewById(R.id.passwordItemRowUser);
				convertView.setTag(viewHolder);
			} else {
				viewHolder = (ViewHolder) convertView.getTag();
			}
			
			viewHolder.title.setText(item.getTitle());
			viewHolder.url.setText(item.getUrl());
			viewHolder.user.setText(item.getUser());

			return convertView;
		}

		public void setData(List<PasswordItem> data) {			
			if (app.isCryptOpen()) {
				this.data = data;
			} else {
				this.data = Collections.emptyList();
			}
			
			notifyDataSetChanged();
		}

		private static class ViewHolder {
			public TextView title;
			public TextView url;
			public TextView user;
		}	
		
	}
	
	public static class PasswordItemFilter extends Filter {
				
		private FpmApplication fpmApp;
		private FpmCryptListAdapter fpmListAdapter;
		volatile private String category = null;
		
		public PasswordItemFilter(FpmApplication fpmApp, FpmCryptListAdapter fpmListAdapter) {
			this.fpmApp = fpmApp;
			this.fpmListAdapter = fpmListAdapter;
		}
		
		protected List<PasswordItem> filterPasswordItems(String category, String title) {
			title = title == null ? StringUtils.EMPTY : title.trim().toUpperCase();
			List<PasswordItem> allItems = getPasswordItems();
			
			android.util.Log.d(TAG, "performFiltering(string=" + title + ", category=" + category + ")");
			
			if ((title == null || StringUtils.isBlank(title)) && category == null) {
				return allItems;
			} else {
				List<PasswordItem> filteredItems = new ArrayList<PasswordItem>(allItems.size());
				for (PasswordItem item : allItems) {
					if (item.getTitle().toUpperCase().contains(title)
							&& acceptCategory(item, category)) {
						filteredItems.add(item);
					}
				}
				return filteredItems;
			}
		}
		
		@Override
		protected FilterResults performFiltering(CharSequence constraintSeq) {
			FilterResults result = new FilterResults();
			result.values = filterPasswordItems(category, constraintSeq == null ? StringUtils.EMPTY : constraintSeq.toString());
			return result;
		}

		@Override
		@SuppressWarnings("unchecked")
		protected void publishResults(CharSequence constraint,
				FilterResults results) {
			fpmListAdapter.setData((List<PasswordItem>) results.values);
		}
		
		/**
		 * Get a list of passwords from the Applications FpmCrypt.
		 * The method may performs unsafe / unsynchronized access
		 * to FpmApplication. publishResults() must re-assert 
		 * the state of FpmCrypt before showing the passwords.
		 * 
		 * @return
		 */
		private List<PasswordItem> getPasswordItems() {
			List<PasswordItem> result = Collections.emptyList();
			try {
				result = fpmApp.getPasswordItems();
			} catch (NullPointerException e) {
				 // If closeCrypt() and getPasswordItems() are running at
				 // the same time null pointer is possible, i try to catch
				 // it here. I don't know why I don't just syhcronize.
				if (fpmApp.isCryptOpen()) {
					throw e;
				}
			}
			return result;
		}

		private boolean acceptCategory(PasswordItem item, String category) {
			return category == null 
					|| (CATEGORY_DEFAULT.equals(category) && item.isDefault())
					|| (category.equals(item.getCategory()));
		}
		
		public String getCategory() {
			return category;
		}

		public void setCategory(String category) {
			this.category = category;
		}
		
	}
	    
}